#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <algorithm>

#include "block_file_accessor.h"



BlockFileAccessor::BlockFileAccessor(const char *path, fileid_t fileid)
: Accessor(fileid)
{
    /* Obtain the size of the file. */

    struct stat buf;

    if (stat(path, &buf))
    {
        _size = 0;
        return;
    }

    /* Allocate memory. */

    _size = buf.st_size;
    _data = (char *) malloc(_size);

    if (_data == NULL)
        return;

    /* Load the file into memory and initialise the size. */

    FILE *fp = fopen(path, "r");

    if (fp == NULL)
    {
        free(_data);
        return;
    }

    _size = fread(_data, sizeof(char), _size, fp);

    fclose(fp);
}

/* Data transfer methods. */

void BlockFileAccessor::fill(Flexpage *flexpage)
{
    offset_t filepos = flexpage->base_offset;
    offset_t addr = flexpage->base_addr;
    offset_t populated_size = std::min(flexpage->size, _size - filepos);

    /* Tag the region with file state. */

    flexpage->region->fill(fileid, filepos);

    /* Fill the region with file content. */

    memcpy((void *) addr, _data + filepos, populated_size);

    /* Pad the flexpage with zero. */

    if (populated_size < flexpage->size)
        memset((void *) (addr + populated_size), 0, flexpage->size - populated_size);
}

void BlockFileAccessor::flush(Flexpage *flexpage)
{
    offset_t addr = flexpage->base_addr;
    offset_t filepos = flexpage->base_offset;
    offset_t populated_size = std::min(flexpage->size, _size - filepos);

    /* Remove the file state tag from the region. */

    flexpage->region->flush();

    /* Copy the populated region to the block memory. */

    memcpy((void *) (_data + filepos), (const void *) addr, populated_size);
}

// vim: tabstop=4 expandtab shiftwidth=4
